/************************************************************************
 * This file is part of AdminCmd.									
 *																		
 * AdminCmd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by	
 * the Free Software Foundation, either version 3 of the License, or		
 * (at your option) any later version.									
 *																		
 * AdminCmd is distributed in the hope that it will be useful,	
 * but WITHOUT ANY WARRANTY; without even the implied warranty of		
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			
 * GNU General Public License for more details.							
 *																		
 * You should have received a copy of the GNU General Public License
 * along with AdminCmd.  If not, see <http://www.gnu.org/licenses/>.
 ************************************************************************/
package be.Balor.Manager.Commands.Player;

import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import be.Balor.Manager.LocaleManager;
import be.Balor.Manager.Commands.CommandArgs;
import be.Balor.Manager.Exceptions.ActionNotPermitedException;
import be.Balor.Manager.Exceptions.PlayerNotFound;
import be.Balor.OpenInv.InventoryManager;
import be.Balor.Tools.CommandUtils.Immunity;
import be.Balor.Tools.CommandUtils.Users;

/**
 * @author Balor (aka Antoine Aflalo)
 * 
 */
public class OpenInventory extends PlayerCommand {

	/**
	 * 
	 */
	public OpenInventory() {
		super("bal_openinv", "admincmd.player.openinv");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.Balor.Manager.Commands.CoreCommand#execute(org.bukkit.command.
	 * CommandSender, be.Balor.Manager.Commands.CommandArgs)
	 */
	@Override
	public void execute(final CommandSender sender, final CommandArgs args)
			throws PlayerNotFound, ActionNotPermitedException {
		if (!Users.isPlayer(sender)) {
			return;
		}
		final String playerName = args.getString(0);
		final Player target = Users.getPlayer(playerName);
		final Player pSender = (Player) sender;
		if (target == null) {
			String world = args.getValueFlag('w');
			if (world == null) {
				world = pSender.getWorld().getName();
			}
			InventoryManager.INSTANCE
					.openOfflineInv(pSender, playerName, world);
			return;
		}
		if (!Immunity.checkImmunity(sender, target)) {
			LocaleManager.sI18n(sender, "insufficientLvl");
			return;
		}

		InventoryManager.INSTANCE.openInv(pSender, target);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.Balor.Manager.Commands.CoreCommand#argsCheck(java.lang.String[])
	 */
	@Override
	public boolean argsCheck(final String... args) {
		return args != null && args.length >= 1;
	}

}
